import fs from 'fs-extra';
import glob from 'glob';
import { table } from 'table';
import findUp from 'find-up';
import _ from 'lodash';
import colorize from 'json-colorizer';
import stringify from 'json-stringify-pretty-compact';
import chalk from 'chalk';
import randomatic from 'randomatic';
import day from 'dayjs';
import co from 'co';
import shell from 'shelljs';
import debug from 'debug';
import inquirer from 'inquirer';
import fuzzy from 'fuzzy';
import emoji from 'node-emoji';
import { dd, dump } from 'dumper.js';
import getStdin from 'get-stdin';
import NodeCache from 'node-cache';
import yargs from 'yargs';
import yParser from 'yargs-parser';
/**
 * Get Zignis internal cache instance
 * @returns {NodeCache}
 */
declare const getInternalCache: () => NodeCache;
/**
 * Get Zignis cache instance by namespace
 * @param {string} namespace
 * @returns {NodeCache}
 */
declare const getCache: (namespace: string) => NodeCache;
/**
 * debug core
 */
declare const debugCore: (...args: any[]) => any;
declare const fileExistsSyncCache: (filePath: any) => any;
interface IHookOption {
    mode?: 'assign' | 'merge' | 'push' | 'replace' | 'group';
    useCache?: boolean;
    include?: boolean | string[];
    exclude?: boolean | string[];
    opts?: any;
}
/**
 * Run hook in all valid plugins and return the combined results.
 * Plugins implement hook in `module.exports`, could be generator function or promise function or non-function
 * For non-function, it will be used as hook data directly, likely to be returned by function
 * @example
 * const hookReturn = await Utils.invokeHook('hook')
 * @param {string} hook Hook name, suggest plugin defined hook include a prefix, e.g. `zhike:hook`
 * @param {string} options Options
 * @param {string} options.mode Hook mode, could be `assign`, `merge`, `push`, `replace`, `group`, default is assign.
 * @param {bool} options.useCache If or not use cached hook result
 * @param {array} options.include set plugins to be used in invoking
 * @param {array} options.exclude set plugins not to be used in invoking, same ones options.exclude take precedence
 * @param {array} options.opts opts will be sent to hook implementation
 */
declare const invokeHook: (hook: string, options?: IHookOption) => Promise<any>;
/**
 * Extend command's sub command, it give other plugins an opportunity to extend it's sub command.
 * So if you want other plugins to extend your sub commands, you can use this util function to replace default `yargs.commandDir`
 * @example
 * exports.builder = function (yargs) {
 *   // The first param could be a/b/c if you want to extend subcommand's subcommand
 *   Utils.extendSubCommand('make', 'zignis', yargs, __dirname)
 * }
 * @param {String} command Current command name.
 * @param {String} module Current plugin name.
 * @param {Object} yargs Yargs reference.
 * @param {String} basePath Often set to `__dirname`.
 */
declare const extendSubCommand: (command: string, module: string, yargs: yargs.Argv<{}>, basePath: string) => void;
/**
 * Get all plugins path mapping.
 * Same name plugins would be overriden orderly.
 * This function also influence final valid commands and configs.
 */
declare const getAllPluginsMapping: () => {
    [propName: string]: string;
};
/**
 * Get application zignis config only.
 */
declare const getApplicationConfig: (cwd?: string | undefined) => any;
/**
 * Format options keys
 *
 * Make compatible of param cases and camel cases
 */
declare const formatRcOptions: (opts: any) => object;
/**
 * Get commbined config from whole environment.
 */
declare const getCombinedConfig: () => {
    [propName: string]: any;
};
/**
 * Print message with format and color.
 * @param {mix} message Message to log
 */
declare const log: (message: any) => void;
/**
 * Print error message, and exit process.
 * @param {mix} message Error message to log
 * @param {string} label Error log label
 * @param {integer} errorCode Error code
 */
declare const error: (message: any, exit?: boolean, errorCode?: number) => void;
/**
 * Print warn message with yellow color.
 * @param {mix} message Error message to log
 */
declare const warn: (message: any, exit?: boolean, errorCode?: number) => void;
/**
 * Print info message with green color.
 * @param {mix} message Error message to log
 */
declare const info: (message: any, exit?: boolean, errorCode?: number) => void;
/**
 * Print success message with green color.
 * @param {mix} message Error message to log
 */
declare const success: (message: any, exit?: boolean, errorCode?: number) => void;
/**
 * Compute md5.
 * @param {string} s
 */
declare const md5: (s: string) => string;
/**
 * Split input by comma and blank.
 * @example
 * const = Utils.splitComma('a, b , c,d')
 * @param {string} input
 * @returns {array} input separated by comma
 */
declare const splitComma: (input: string) => string[];
/**
 * Split input by a specific char and blank.
 * @example
 * const = Utils.splitByChar('a, b , c=d', '=')
 * @param {string} input
 * @returns {array} input separated by comma
 */
declare const splitByChar: (input: string, char: string) => string[];
/**
 * Print a simple table.
 * A table style for `zignis status`, if you don't like this style, can use Utils.table
 * @param {array} columns Table columns
 * @param {string} caption Table caption
 * @param {object} borderOptions Border options
 */
declare const outputTable: (columns: string[][], caption: string, borderOptions?: {}) => void;
/**
 * Parse packages from yargs option
 * @param {*} input yarns option input, could be string or array
 * @returns {array} Package list
 */
declare const parsePackageNames: (input: string | string[]) => string[];
/**
 * Load any package's package.json
 * @param {string} pkg package name
 * @param {array} paths search paths
 */
declare const loadPackageInfo: (pkg?: string | undefined, paths?: never[]) => any;
/**
 * Load core package.json
 */
declare const loadCorePackageInfo: () => any;
/**
 * Execute command, because npm install running info can not be catched by shelljs, temporarily use this one
 * @param {string} command Command to exec
 * @param {object} options Options stdio default is [0, 1, 2]
 */
declare const exec: (command: string, options?: any) => any;
/**
 * Get current node env setting
 *
 * You can change the node-env-key in command args or zignis rc file
 */
declare const getNodeEnv: () => string;
/**
 * Shortcut for checking if or not current env is production
 */
declare const isProduction: () => boolean;
/**
 * Shortcut for checking if or not current env is development
 */
declare const isDevelopment: () => boolean;
/**
 * Sleep a while of ms
 * @param {integer} ms
 */
declare const sleep: (ms: number) => Promise<unknown>;
declare const delay: (ms: number) => Promise<unknown>;
/**
 * Zignis utils functions and references to common modules.
 * @module Utils
 */
export { 
/** [lodash](https://www.npmjs.com/package/lodash) reference, check [doc](https://lodash.com/docs). */
_, 
/** [chalk](https://www.npmjs.com/package/chalk) reference */
chalk, 
/** [table](https://www.npmjs.com/package/table) reference */
table, 
/** [day.js](https://www.npmjs.com/package/dayjs) reference, check [api](https://github.com/iamkun/dayjs/blob/HEAD/docs/en/API-reference.md) documentation. */
day, 
/** [json-colorizer](https://www.npmjs.com/package/json-colorizer) reference */
colorize, 
/** [json-stringify-pretty-compact](https://www.npmjs.com/package/json-stringify-pretty-compact) reference. */
stringify, 
/** [glob](https://www.npmjs.com/package/glob) reference. */
glob, 
/** [find-up](https://www.npmjs.com/package/find-up) reference. */
findUp, 
/** [co](https://www.npmjs.com/package/co) reference. */
co, 
/** [shelljs](https://www.npmjs.com/package/shelljs) reference. */
shell, 
/** [debug](https://www.npmjs.com/package/debug) reference. */
debug, 
/** [fuzzy](https://www.npmjs.com/package/fuzzy) reference. */
fuzzy, 
/** [inquirer](https://www.npmjs.com/package/inquirer) reference, with autocomplete plugin */
inquirer, 
/** [fs-extra](https://www.npmjs.com/package/fs-extra) reference */
fs, 
/** [randomatic](https://www.npmjs.com/package/randomatic) reference */
randomatic, 
/** [node-emoji](https://www.npmjs.com/package/node-emoji) reference */
emoji, 
/** [get-stdin](https://www.npmjs.com/package/get-stdin) reference */
getStdin, 
/** [node-cache](https://www.npmjs.com/package/node-cache) reference */
NodeCache, 
/** [yargs-parser](https://www.npmjs.com/package/yargs-parser) reference */
yParser, md5, delay, splitComma, splitByChar, log, warn, info, success, dd, dump, error, outputTable, invokeHook, extendSubCommand, getAllPluginsMapping, getCombinedConfig, getApplicationConfig, parsePackageNames, loadPackageInfo, loadCorePackageInfo, exec, sleep, getInternalCache, getCache, getNodeEnv, isProduction, isDevelopment, fileExistsSyncCache, debugCore, formatRcOptions };
